#include "render.h"

Render::Render()
{
	cam = new Camera();
}

Render::~Render()
{
}

bool Render::Init(HWND hwnd)
{
	memset(&pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));
	pfd =				// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		16,										// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};

	hdc = GetDC(hwnd);
	int pfdNum = ChoosePixelFormat(hdc, &pfd);
	if (pfdNum != 0){
		if (SetPixelFormat(hdc, pfdNum, &pfd) == FALSE){
			DWORD dwErrorCode = GetLastError();
			return false;
		}
	}
	hglrc = wglCreateContext(hdc);
	HGLRC tempContext = wglCreateContext(hdc);
	if (!wglMakeCurrent(hdc, hglrc))
		return false;
	GLenum err = glewInit();
	if (GLEW_OK != err)
		return false;
	models = new Model[1];
	cam->Init();
	delay = 1;
	rate = 0.1f;
	return true;
}

void Render::StartCoordinate()
{
	models[0].Scalling(vec3(0.5f, 0.5f, 0.5f));
	cam->TranslateTo(vec3(0.0f, 0.0f, 0.0f));
	cam->Translate(vec3(0.0f, 0.0f, models[0].ms.maxP.z*2.0f));
	dz = models[0].ms.maxP.z - models[0].ms.lenght * 0.05;
	dz2 = models[0].ms.maxP.z;
	rate = dz2 - dz;
	cutWidth = 0.1;
}


int tic = 0;
float angle = 0.001f;
float an = angle;
bool front = true;

void Render::MatrixUpdate()
{
	tic++;
	float end = models[0].ms.lenght/2 + models[0].ms.position.z;
	if (tic == delay){
		if (front == true){
			dz += rate;
			dz2 += rate;			
		}
		else{
			dz -= rate;
			dz2 -= rate;
		}
		tic = 0;
	}
	if (dz2 > models[0].ms.maxP.z + models[0].ms.lenght){
		dz2 = models[0].ms.maxP.z;
		dz = models[0].ms.maxP.z - models[0].ms.lenght * 0.05;
	}
	GLuint cut = glGetUniformLocation(program, "cut");
	GLuint range = glGetUniformLocation(program, "range");
	glUniform1f(cut, dz - abs(dz-dz2)/2);
	glUniform1f(range, abs(dz - dz2)*cutWidth);
}

void Render::Run()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glUseProgram(program);	
	GLuint p = glGetUniformLocation(program, "lightSource.position");
	vec4 pL = vec4(0.0f, 0.0f, -models[0].ms.maxP.z*2.0f, 1.0);
	glUniform4fv(p, 1, &pL[0]);
	vec4 diff = vec4(2.0f,2.0f,2.0f,1.0f);
	p = glGetUniformLocation(program, "lightSource.diffuse");
	glUniform4fv(p, 1, &diff[0]);
	p = glGetUniformLocation(program, "material.diffuse");
	glUniform4fv(p, 1, &diff[0]);
	
	for (auto i = 0; i < 1; i++)
	{
		models[i].Draw(Unif_mat,cam);
	}
	SwapBuffers(hdc);
	MatrixUpdate();
}

void Render::Shutdown(HWND hwnd)
{
	delete[] models;
	glDeleteProgram(program);
	glUseProgram(0);
	glDeleteProgram(program);	
	if (hdc != NULL){
		wglMakeCurrent(NULL, NULL);
		wglDeleteContext(hglrc);
		hglrc = NULL;
	}
	if (hdc != NULL){
		ReleaseDC(hwnd, hdc);
		hdc = NULL;
	}
}

void Render::PrepareScene()
{
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations							// Enables Depth Testing
	shaders = new Shader[2];
	shaders[0].LoadShader("Shaders/ShVertex.txt", VERTEX_SH);
	shaders[1].LoadShader("Shaders/ShFragment.txt", FRAGMENT_SH);
	program = glCreateProgram();
	shaders[0].AttachShader(&program);
	shaders[1].AttachShader(&program);	
	glLinkProgram(program);	
	int link_ok, LogLen;
	glGetProgramiv(program, GL_LINK_STATUS, &link_ok);
	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &LogLen);
	if (LogLen > 0){
		std::vector<char> FragmentShaderErrorMessage(LogLen + 1);
		GLchar *info = new GLchar[100];
		glGetShaderInfoLog(program, LogLen, NULL, &FragmentShaderErrorMessage[0]);
	}
	if (!link_ok)
	{
		MessageBox(0, "Lol", "lol", 0);
	}
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	pointLight.position = vec4(3.0f, 3.0f, 3.0f, 1.0f);
	pointLight.ambient = vec4(0.1f, 0.1f, 0.1f, 1.0f);
	pointLight.diffuse = vec4(1.0f, 1.0f, 1.0f, 1.0f);
	pointLight.specular = vec4(1.0f, 1.0f, 1.0f, 1.0f);
	pointLight.attenuation = vec3(0.5f, 0.0f, 0.02f);
}

void Render::SetData()
{
	Unif_mat = glGetUniformLocation(program, "matrix");
	vec3 apple1[3] = { vec3(0.0f, 1.0f, 0.0f), vec3(0.58f, 0.29f, 0.0f), vec3(0.0f, 1.0f, 0.0f) };
	vec3 apple2[3] = { vec3(0.0f, 0.8f, 0.0f), vec3(0.0f, 0.8f, 0.0f), vec3(0.0f, 0.8f, 0.0f) };
	vec3 *colors[2] = { apple1, apple2 };
	for (auto i = 0; i < 1; i++)
	{
		if (models[i].fileName == "" || models[i].fileName.length() < 5)
			continue;
		models[i].SetAttributeLocation(program, "coord", 0);
		models[i].SetAttributeLocation(program, "color", 1);
		models[i].SetAttributeLocation(program, "texture", 2);
		models[i].SetAttributeLocation(program, "normal", 3);
		if (models[i].LoadModel() == true){
			models[i].Init(program,true);
		}
	}
	cam->SetPosition(vec3(0.0f, 0.0f, 0.0f));
	cam->Init();
	StartCoordinate();
}

void Render::SetModelFile(string name)
{
	if (models != nullptr)
		delete[] models;
	models = new Model[2];
	models[0].fileName = name;
	models[0].modelName = "object1";
	SetData();
}

Model* Render::GetModelByName(string name)
{
	if (models == nullptr)
		return nullptr;
	for (auto i = 0; i < 1; i++)
	{
		if (models[i].modelName == name)
			return &models[i];
	}
	return nullptr;
}
