#pragma once
#include "dependences.h"
#include "Camera.h"
struct Vertex
{
	GLfloat	x;
	GLfloat y;
	GLfloat z;

};

struct ColorRGB
{
	GLfloat r;
	GLfloat g;
	GLfloat b;
};

struct Index
{
	unsigned int i1;
	unsigned int i2;
	unsigned int i3;
	//unsigned int t1;
	//unsigned int t2;
};

struct TextureCoord
{
	GLfloat u;
	GLfloat v;
};

struct VertexTexture
{
	vec3 vert;
	vec2 text;
	vec3 norm;
};

struct ModelSize
{
	Vertex position;
	float width;
	float height;
	float lenght;
	vec3 maxP;
	vec3 minP;
};

struct PointLight
{
	vec4  position;
	vec4  ambient;
	vec4  diffuse;
	vec4  specular;
	vec3  attenuation;
};

static void PointLightSetup(GLuint program, const PointLight &light)
{
	//  
	glUniform4fv(glGetUniformLocation(program, "light.position"), 1, &light.position[0]);
	glUniform4fv(glGetUniformLocation(program, "light.ambient"), 1, &light.ambient[0]);
	glUniform4fv(glGetUniformLocation(program, "light.diffuse"), 1, &light.diffuse[0]);
	glUniform4fv(glGetUniformLocation(program, "light.specular"), 1, &light.specular[0]);
	glUniform3fv(glGetUniformLocation(program, "light.attenuation"), 1, &light.attenuation[0]);
}

struct Material
{
	GLuint texture;
	vec4  ambient;
	vec4  diffuse;
	vec4  specular;
	vec4  emission;
	float shininess;
};

static void MaterialSetup(GLuint program, const Material &material)
{
	//  
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, material.texture);
	glUniform1i(glGetUniformLocation(program, "material.texture"), 0);

	//  
	glUniform4fv(glGetUniformLocation(program, "material.ambient"), 1, &material.ambient[0]);
	glUniform4fv(glGetUniformLocation(program, "material.diffuse"), 1, &material.diffuse[0]);
	glUniform4fv(glGetUniformLocation(program, "material.specular"), 1, &material.specular[0]);
	glUniform4fv(glGetUniformLocation(program, "material.emission"), 1, &material.emission[0]);

	glUniform1fv(glGetUniformLocation(program, "material.shininess"), 1, &material.shininess);
}


static void CameraSetup(GLuint program, Camera *cam, const mat4 Projection, const mat4 &model)
{
	//   
	mat4 view = cam->GetMatrix();
	mat4 viewProjection = Projection * view;
	mat3 normal = transpose(mat3(inverse(model)));

	//     
	glUniformMatrix4fv(glGetUniformLocation(program, "transform.model"), 1, GL_TRUE, &model[0][0]);
	glUniformMatrix4fv(glGetUniformLocation(program, "transform.viewProjection"), 1, GL_TRUE, &viewProjection[0][0]);
	glUniformMatrix3fv(glGetUniformLocation(program, "transform.normal"), 1, GL_TRUE, &normal[0][0]);

	//    ()   
	glUniform3fv(glGetUniformLocation(program, "transform.viewPosition"), 1, &cam->GetPosition()[0]);
}
